//
// MarriageAlg.java
//

import java.util.*;

public class MarriageAlg
{
    public static final int NUM = 10;
    public static Random RNG;
    public Student students[];
    public int presList[];

    MarriageAlg()
    {
	super();

	// create RNG
	RNG = new Random();

	// presentation order
	presList = new int[NUM];
	for (int i = 0; i < NUM; ++i)
	    presList[i] = -1;
    }

    // create the set of students with random preferences
    public void createStudents()
    {
	students = new Student[NUM];
	for (int i = 0; i < NUM; ++i) {
	    students[i] = new Student();
	    students[i].randomPreference();
	}
    }

    // perform the marriage algorithm
    public void runAlg()
    {
	int first = RNG.nextInt(10);
	boolean done = false;
	//System.out.println("Random first student is: " + first);

	int current = first;
	while (!done) {
	    // put student in slot according to preference
	    int[] pl = students[current].getPreferences();
	    for (int i = 0; i < NUM; ++i) {
		if (presList[pl[i]] == -1) {
		    // slot is open!
		    presList[pl[i]] = current;
		    ++current;
		    current = current % NUM;
		    if (current == first) done = true;
		    break;
		}
	    }
	}
    }

    public void printStudentPref()
    {
	System.out.println("Student preferences.");
	for (int i = 0; i < NUM; ++i) {
	    System.out.print("Student " + i + ":");
	    students[i].printPreferences();
	    System.out.println("");
	}
    }

    public void printList()
    {
	System.out.println("Order of presentations is:");
	for (int i = 0; i < NUM; ++i)
	    System.out.println("Student " + presList[i]);
    }

    public void calcMetric()
    {
	//
	int metric = 0;
	for (int i = 0; i < NUM; ++i) {
	    metric += students[presList[i]].calcMetric(i);
	}
	//System.out.println("Metric: " + metric);
	System.out.println(metric);
    }

    public static void main(String args[])
    {
	for (int i = 0; i < 100; ++i) {
	    MarriageAlg ma = new MarriageAlg();

	    ma.createStudents();
	    ma.runAlg();
	    //ma.printStudentPref();
	    //ma.printList();	
	    ma.calcMetric();
	}
    }
}
